# Generated by Django 6.0.2 on 2026-03-03 19:25

from decimal import Decimal
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('terminusgps_notifier', '0002_alter_wialonnotification_act'),
    ]

    operations = [
        migrations.AlterField(
            model_name='terminusgpsnotifiercustomer',
            name='tax_rate',
            field=models.DecimalField(decimal_places=4, default=Decimal('0.0825'), help_text='Enter a tax rate as a decimal.', max_digits=12),
        ),
    ]
