# Generated by Django 6.0.2 on 2026-02-20 18:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('terminusgps_notifier', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='wialonnotification',
            name='act',
            field=models.JSONField(blank=True, default=list),
        ),
    ]
